from .executors import *


class ExecutorFactory:
    def __init__(self, engine_version="13"):
        self.engine_version = engine_version.split('.')[0]

    def build(self, database_name, endpoint, user_name, ssm_db_password_path):
        try:
            klass = globals()["Executor_v{0}".format(self.engine_version)]
            return klass(database_name, endpoint, user_name, ssm_db_password_path)
        except KeyError:
            raise Exception(f"Postgres engine {self.engine_version} is not supported, please provide an executor")



from .executor13 import _Executor13


# Executor for postgres 14
class _Executor14(_Executor13):
    def __init__(self, database_name,  endpoint, user_name, ssm_db_password_path):
        super().__init__(database_name, endpoint, user_name, ssm_db_password_path)

    def __grant_read_only_roles(self, user_name):
        return "GRANT pg_read_all_data TO \"{0}\";".format(user_name)

    def __log_grant_read_only_roles(self, user_name):
        return "L'utilisateur {0} a herite des droits read only sur toutes les tables, vues et sequences \n".format(user_name)

    def __execute_grant_read_only_roles(self, user_name):
        self.__cursor.execute(self.__grant_read_only_roles(user_name))
        return self.__log_grant_read_only_roles(user_name)

    def setup_hawking(self, hawking_user_name):
        log_grant_read_only = self.__execute_grant_read_only_roles(hawking_user_name)
        self.__logs.extend(list(filter(None, [log_grant_read_only])))


from .ssm import fetch_password_from_ssm
import pg8000


def get_connection(db_name, endpoint, user_name, ssm_db_password_path):
    """
        Method to establish the connection.
    """
    try:
        print("Connecting to database")
        # Read the environment variables to get DB EndPoint
        db_password = fetch_password_from_ssm(ssm_db_password_path)

        # Establishes the connection with the server using the token generated as password
        conn = pg8000.connect(
            host=endpoint,
            user=user_name,
            database=db_name,
            password=db_password
        )
        conn.autocommit = True
        return conn
    except Exception as e:
        print("While connecting failed due to :{0}".format(str(e)))
        return None



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['lcdp_postgres_utils',
 'lcdp_postgres_utils.executors',
 'lcdp_postgres_utils.utils']

package_data = \
{'': ['*']}

install_requires = \
['boto3>=1.26.55,<2.0.0', 'pg8000>=1.29.4,<2.0.0']

setup_kwargs = {
    'name': 'lcdp-postgres-utils',
    'version': '1.0.4.dev21',
    'description': 'Postgres Utils to create users, databases, functions, ...',
    'long_description': 'None',
    'author': 'Le Comptoir Des Pharmacies',
    'author_email': 'webmaster@lecomptoirdespharmacies.fr',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
